
//   Copyright 2022 Esri
//   Licensed under the Apache License, Version 2.0 (the "License");
//   you may not use this file except in compliance with the License.
//   You may obtain a copy of the License at
//
//   https://www.apache.org/licenses/LICENSE-2.0
//
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS,
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and
//   limitations under the License.

using System;
using System.Collections.Generic;
using System.Text;
using Esri.ArcGISRuntime.Geometry;
using Esri.ArcGISRuntime.Mapping;
using System.ComponentModel;
using System.Runtime.CompilerServices;

using Esri.ArcGISRuntime.Data;
using Esri.ArcGISRuntime.Mapping.Labeling;
using Esri.ArcGISRuntime.Symbology;
using System.Diagnostics; 
using System.Drawing;

namespace StyleAFeatureLayer
{

    class MapViewModel : INotifyPropertyChanged
    {

        // Application constants used to connect to data and resources.
        static Uri parksAndOpenSpaces = new Uri("https://services3.arcgis.com/GVgbJbqm8hXASVYi/arcgis/rest/services/Parks_and_Open_Space/FeatureServer/0");
        static Uri trails = new Uri("https://services3.arcgis.com/GVgbJbqm8hXASVYi/arcgis/rest/services/Trails/FeatureServer/0");
        static Uri trailheads = new Uri("https://services3.arcgis.com/GVgbJbqm8hXASVYi/arcgis/rest/services/Trailheads/FeatureServer/0");
        static Uri trailheadImage = new Uri("https://static.arcgis.com/images/Symbols/NPS/npsPictograph_0231b.png");

        public MapViewModel()
        {
            SetupMap();
        }

        public event PropertyChangedEventHandler? PropertyChanged;
        protected void OnPropertyChanged([CallerMemberName] string propertyName = "")
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }


        private Map? _map;
        public Map? Map
        {
            get { return _map; }
            set
            {
                _map = value;
                OnPropertyChanged();
            }
        }

        private FeatureLayer AddFeatureLayer(Uri featureServiceUri)
        {
            // Create a service feature table from a Uri.
            ServiceFeatureTable serviceFeatureTable = new ServiceFeatureTable(featureServiceUri);

            // Create a feature layer from the service feature table.
            FeatureLayer featureLayer = new FeatureLayer(serviceFeatureTable);

            // Add the feature layer to the maps operational layers collection.
            Map?.OperationalLayers.Add(featureLayer);

            // Return the feature layer.
            return featureLayer;
        }

        private void AddOpenSpaceLayer()
        {
            // Create a parks and open spaces feature layer and add it to the map view.
            FeatureLayer featureLayer = AddFeatureLayer(parksAndOpenSpaces);

            // Create fill symbols.
            SimpleFillSymbol purpleFillSymbol = new SimpleFillSymbol(SimpleFillSymbolStyle.Solid, Color.Purple, null);
            SimpleFillSymbol greenFillSymbol = new SimpleFillSymbol(SimpleFillSymbolStyle.Solid, Color.Green, null);
            SimpleFillSymbol blueFillSymbol = new SimpleFillSymbol(SimpleFillSymbolStyle.Solid, Color.Blue, null);
            SimpleFillSymbol redFillSymbol = new SimpleFillSymbol(SimpleFillSymbolStyle.Solid, Color.Red, null);

            // Create a unique value for natural areas, regional open spaces, local parks, and regional recreation parks.
            UniqueValue naturalAreas = new UniqueValue("Natural Areas", "Natural Areas", purpleFillSymbol, "Natural Areas");
            UniqueValue regionalOpenSpace = new UniqueValue("Regional Open Space", "Regional Open Space", greenFillSymbol, "Regional Open Space");
            UniqueValue localPark = new UniqueValue("Local Park", "Local Park", blueFillSymbol, "Local Park");
            UniqueValue regionalRecreationPark = new UniqueValue("Regional Recreation Park", "Regional Recreation Park", redFillSymbol, "Regional Recreation Park");

            // Create a unique value renderer to display these values.
            UniqueValueRenderer openSpacesUniqueValueRenderer = new UniqueValueRenderer
                (new List<string>() { 
                    "TYPE" 
                }, 
                new List<UniqueValue>() { 
                    naturalAreas, regionalOpenSpace, localPark, regionalRecreationPark 
                }, 
                "Open Spaces", 
                null);

            // Assign the unique value renderer to the open spaces layer.
            featureLayer.Renderer = openSpacesUniqueValueRenderer;

            // Set the layer opacity to semi-transparent.
            featureLayer.Opacity = 0.2;
        }

        private void AddTrailsLayer()
        {
            // Create a trails feature layer and add it to the map view.
            FeatureLayer featureLayer = AddFeatureLayer(trails);

            // Create simple line symbols.
            SimpleLineSymbol firstClassSymbol = new SimpleLineSymbol(SimpleLineSymbolStyle.Solid, Color.Purple, 3.0);
            SimpleLineSymbol secondClassSymbol = new SimpleLineSymbol(SimpleLineSymbolStyle.Solid, Color.Purple, 4.0);
            SimpleLineSymbol thirdClassSymbol = new SimpleLineSymbol(SimpleLineSymbolStyle.Solid, Color.Purple, 5.0);
            SimpleLineSymbol fourthClassSymbol = new SimpleLineSymbol(SimpleLineSymbolStyle.Solid, Color.Purple, 6.0);
            SimpleLineSymbol fifthClassSymbol = new SimpleLineSymbol(SimpleLineSymbolStyle.Solid, Color.Purple, 7.0);

            // Create five class breaks.
            ClassBreak firstClassBreak = new ClassBreak("Under 500", "0 - 500", 0.0, 500.0, firstClassSymbol);
            ClassBreak secondClassBreak = new ClassBreak("501 to 1000", "501 - 1000", 501.0, 1000.0, secondClassSymbol);
            ClassBreak thirdClassBreak = new ClassBreak("1001 to 1500", "1001 - 1500", 1001.0, 1500.0, thirdClassSymbol);
            ClassBreak fourthClassBreak = new ClassBreak("1501 to 2000", "1501 - 2000", 1501.0, 2000.0, fourthClassSymbol);
            ClassBreak fifthClassBreak = new ClassBreak("2001 to 2300", "2001 to 2300", 2001.0, 2300.0, fifthClassSymbol);
            List<ClassBreak> elevationBreaks = new List<ClassBreak>() { firstClassBreak, secondClassBreak, thirdClassBreak, fourthClassBreak, fifthClassBreak };

            // Create and assign a class breaks renderer to the feature layer.
            ClassBreaksRenderer elevationClassBreaksRenderer = new ClassBreaksRenderer("ELEV_GAIN", elevationBreaks);
            featureLayer.Renderer = elevationClassBreaksRenderer;

            // Set the layer opacity to semi-transparent.
            featureLayer.Opacity = 0.75;
        }

        private void AddBikeOnlyTrailsLayer()
        {
            // Create a trails feature layer and add it to the map view.
            FeatureLayer featureLayer = AddFeatureLayer(trails);

            // Write a definition expression to filter for trails that permit the use of bikes.
            featureLayer.DefinitionExpression = "USE_BIKE = 'Yes'";

            // Create and assign a simple renderer to the feature layer.
            SimpleLineSymbol bikeTrailSymbol = new SimpleLineSymbol(SimpleLineSymbolStyle.Dot, Color.Blue, 2.0);
            SimpleRenderer bikeTrailRenderer = new SimpleRenderer(bikeTrailSymbol);
            featureLayer.Renderer = bikeTrailRenderer;
        }

        private void AddNoBikeTrailsLayer()
        {
            // Create a trails feature layer and add it to the map view.
            FeatureLayer featureLayer = AddFeatureLayer(trails);

            // Write a definition expression to filter for trails that don't permit the use of bikes.
            featureLayer.DefinitionExpression = "USE_BIKE = 'No'"; 

            // Create and assign a simple renderer to the feature layer.
            SimpleLineSymbol noBikeTrailSymbol = new SimpleLineSymbol(SimpleLineSymbolStyle.Dot, Color.Red, 2.0);
            SimpleRenderer noBikeTrailRenderer = new SimpleRenderer(noBikeTrailSymbol);
            featureLayer.Renderer = noBikeTrailRenderer;
        }

        private LabelDefinition MakeLabelDefinition(string labelAttribute)
        {
            // Create a new text symbol.
            TextSymbol labelTextSymbol  = new TextSymbol
            {
                Color = Color.White,
                Size = 12.0,
                HaloColor = Color.Red,
                HaloWidth = 1.0,
                FontFamily = "Arial",
                FontStyle = FontStyle.Italic,
                FontWeight = FontWeight.Normal
            };

            // Create a new Arcade label expression based on the field name.
            ArcadeLabelExpression labelExpression = new ArcadeLabelExpression("$feature." + labelAttribute);

            // Create and return the label definition.
            return new LabelDefinition(labelExpression, labelTextSymbol );
        }

        private void AddTrailheadsLayer()
        {
            // Create a trailheads feature layer and add it to the map view.
            FeatureLayer featureLayer = AddFeatureLayer(trailheads);

            // Create a new picture marker symbol that uses the trailhead image.
            PictureMarkerSymbol pictureMarkerSymbol = new PictureMarkerSymbol(trailheadImage);
            pictureMarkerSymbol.Height = 18.0;
            pictureMarkerSymbol.Width = 18.0;

            // Create a new simple renderer based on the picture marker symbol.
            SimpleRenderer simpleRenderer = new SimpleRenderer(pictureMarkerSymbol);

            // Set the feature layer's renderer and enable labels.
            featureLayer.Renderer = simpleRenderer;
            featureLayer.LabelsEnabled = true;

            // Create the label definition and pass the "TRL_NAME" attribute to label features by trail name.
            LabelDefinition trailHeadsDefinition = MakeLabelDefinition("TRL_NAME");

            // Add the label definition to the layer's label definition collection.
            featureLayer.LabelDefinitions.Add(trailHeadsDefinition);
        }

        private void SetupMap()
        {
            // Create a new map with a 'topographic vector' basemap.
            Map = new Map(BasemapStyle.ArcGISTopographic);

            AddOpenSpaceLayer();

            AddTrailsLayer();

            AddNoBikeTrailsLayer();
            
            AddBikeOnlyTrailsLayer();

            AddTrailheadsLayer();

        }

    }
}


